@@ -1,6 +1,6 @@
 ;;; ebib-filters.el --- Part of Ebib, a BibTeX database manager  -*- lexical-binding: nil -*-
 
-;; Copyright (c) 2003-2019 Joost Kremers
+;; Copyright (c) 2003-2022 Joost Kremers
 ;; All rights reserved.
 
 ;; Author: Joost Kremers <joostkremers@fastmail.fm>
@@ -44,7 +44,7 @@
 (require 'ebib-keywords)
 (require 'ebib-db)
 
-(defgroup ebib-filters nil "Filter settings for Ebib" :group 'ebib)
+(defgroup ebib-filters nil "Filter settings for Ebib." :group 'ebib)
 
 (defcustom ebib-filters-display-as-lisp nil
   "If set, display filters as Lisp expressions."
@@ -56,6 +56,16 @@
   :group 'ebib-filters
   :type 'boolean)
 
+(defcustom ebib-filters-include-crossref nil
+  "If set, include field values from cross-referenced entries.
+By default, if an entry inherits field values from a
+cross-referenced entry, those field values are not checked
+against the filter.  If this option is set, those field values
+are checked, so that entries for which the filter matches on a
+cross-referenced field are included in the results as well."
+  :group 'ebib-filters
+  :type 'boolean)
+
 (defcustom ebib-filters-default-file "~/.emacs.d/ebib-filters"
   "File for saving filters."
   :group 'ebib-filters
@@ -84,21 +94,21 @@ a logical `not' is applied to the selection."
                     ((string= field "=type=")
                      (completing-read prompt (ebib--list-entry-types dialect t) nil t nil 'ebib--filters-history))
                     ((cl-equalp field "keywords")
-                     (completing-read prompt (ebib--keywords-for-database ebib--cur-db)  nil nil nil 'ebib--keywords-history))
+                     (completing-read prompt ebib--keywords-completion-list  nil nil nil 'ebib--keywords-history))
                     (t
                      (read-string prompt nil 'ebib--filters-history)))))
       (ebib--execute-when
-        ((filtered-db)
+        (filtered-db
          (ebib-db-set-filter `(,bool ,(ebib-db-get-filter ebib--cur-db)
-                                 ,(if (>= not 0)
-                                      `(contains ,field ,regexp)
-                                    `(not (contains ,field ,regexp))))
-                         ebib--cur-db))
-        ((real-db)
+                                     ,(if (>= not 0)
+                                          `(contains ,field ,regexp)
+                                        `(not (contains ,field ,regexp))))
+                             ebib--cur-db))
+        (real-db
          (ebib-db-set-filter (if (>= not 0)
-                             `(contains ,field ,regexp)
-                           `(not (contains ,field ,regexp)))
-                         ebib--cur-db))))))
+                                 `(contains ,field ,regexp)
+                               `(not (contains ,field ,regexp)))
+                             ebib--cur-db))))))
 
 ;; The filters keymap
 (eval-and-compile
@@ -119,16 +129,17 @@ a logical `not' is applied to the selection."
   (define-key ebib-filters-map "S" 'ebib-filters-save-filters)
   (define-key ebib-filters-map "v" 'ebib-filters-view-filter)
   (define-key ebib-filters-map "V" 'ebib-filters-view-all-filters)
-  (define-key ebib-filters-map "w" 'ebib-filters-write-to-file))
+  (define-key ebib-filters-map "w" 'ebib-filters-write-to-file)
+  (define-key ebib-filters-map "x" 'ebib-filters-toggle-crossref))
 
 (defun ebib-filters-view-filter ()
   "Display the currently active filter in the minibuffer."
   (interactive)
   (ebib--execute-when
-    ((filtered-db)
+    (filtered-db
      (message (ebib--filters-pp-filter (ebib-db-get-filter ebib--cur-db))))
-    ((default)
-     (error "[Ebib] No filter is active"))))
+    (default
+      (error "[Ebib] No filter is active"))))
 
 (defun ebib-filters-view-all-filters ()
   "Display all filters in a *Help* buffer."
@@ -240,11 +251,13 @@ See `ebib--filters-run-filter'.")
     (eval
      `(cl-macrolet ((contains (field regexp)
                               `(ebib--search-in-entry ,regexp ebib-entry ,(unless (cl-equalp field "any") field))))
-        (delq nil (mapcar (lambda (key)
-                            (let ((ebib-entry (ebib-db-get-entry key db 'noerror)))
-                              (when ,filter
-                                key)))
-                          (ebib-db-list-keys db)))))))
+        (seq-filter (lambda (key)
+                      (let ((ebib-entry (if ebib-filters-include-crossref
+                                            (ebib-get-entry key db 'noerror 'xref)
+                                          (ebib-db-get-entry key db 'noerror))))
+                        (when ,filter
+                          key)))
+                    (ebib-db-list-keys db))))))
 
 (defun ebib--filters-pp-filter (filter)
   "Convert FILTER into a string suitable for displaying.
@@ -279,7 +292,7 @@ return value is also nil."
 
 (defun ebib--filters-load-file (file &optional overwrite)
   "Load filters from FILE.
-If OVERWRITE in non-NIL, the existing filters are discarded.
+If OVERWRITE in non-nil, the existing filters are discarded.
 Otherwise the new filters are added to the existing ones, unless
 there is a name conflict."
   (when (file-readable-p file)
@@ -290,8 +303,8 @@ there is a name conflict."
                      (forward-char -1)
                      (read (current-buffer)))))
         (if (not (listp flist))
-            (ebib--log 'warning "No filters found in %s\n" file)
-          (ebib--log 'log "%s: Loading filters from file %s.\n" (format-time-string "%d %b %Y, %H:%M:%S") file)
+            (ebib--log 'warning "No filters found in `%s'\n" file)
+          (ebib--log 'log "Loading filters from file `%s'.\n" file)
           (if overwrite
               (setq ebib--filters-alist nil))
           (mapc (lambda (filter)
@@ -328,23 +341,23 @@ this file is deleted."
 (defun ebib--filters-add-filter (filter name &optional overwrite)
   "Add FILTER under NAME in `ebib--filters-alist'.
 If a filter with NAME already exists, the filter is not added,
-unless OVERWRITE is non-NIL."
+unless OVERWRITE is non-nil."
   (if (ebib--filters-exists-p name)
       (if overwrite
           (setcdr (ebib--filters-get-filter name) (list filter))
-        (ebib--log 'message "Filter name conflict: \"%s\".\n" name))
+        (ebib--log 'message "Filter name conflict: `%s'.\n" name))
     (push (list name filter) ebib--filters-alist)))
 
 (defun ebib--filters-get-filter (name &optional noerror)
   "Return the filter record corresponding to NAME.
 Return a list (NAME FILTER) if found.  If there is no
-filter named NAME, raise an error, unless NOERROR is non-NIL."
+filter named NAME, raise an error, unless NOERROR is non-nil."
   (or (assoc-string name ebib--filters-alist ebib-filters-ignore-case)
       (unless noerror
         (error "[Ebib] Invalid filter %s" name))))
 
 (defun ebib--filters-exists-p (name)
-  "Return non-NIL if a filter with NAME already exists."
+  "Return non-nil if a filter with NAME already exists."
   (assoc-string name ebib--filters-alist ebib-filters-ignore-case))
 
 ;; Special filters
