@@ -1,6 +1,6 @@
 ;;; ebib-reading-list.el --- Part of Ebib, a BibTeX database manager  -*- lexical-binding: t -*-
 
-;; Copyright (c) 2003-2019 Joost Kremers
+;; Copyright (c) 2003-2022 Joost Kremers
 ;; All rights reserved.
 
 ;; Author: Joost Kremers <joostkremers@fastmail.fm>
@@ -39,6 +39,7 @@
 (require 'ebib-utils)
 
 (declare-function org-get-todo-state "org.el")
+(declare-function org-todo "org.el")
 
 (defgroup ebib-reading-list nil "Settings for the reading list." :group 'ebib)
 
@@ -69,12 +70,12 @@ whether an entry is on the reading list or not."
   :type '(string "Reading list item template"))
 
 (defcustom ebib-reading-list-template-specifiers '((?K . ebib-reading-list-create-org-identifier)
-                                               (?T . ebib-create-org-title)
-                                               (?M . ebib-reading-list-todo-marker)
-                                               (?L . ebib-create-org-link)
-                                               (?F . ebib-create-org-file-link)
-                                               (?D . ebib-create-org-doi-link)
-                                               (?U . ebib-create-org-url-link))
+                                                   (?T . ebib-create-org-title)
+                                                   (?M . ebib-reading-list-todo-marker)
+                                                   (?L . ebib-create-org-link)
+                                                   (?F . ebib-create-org-file-link)
+                                                   (?D . ebib-create-org-doi-link)
+                                                   (?U . ebib-create-org-url-link))
   "Specifiers used in `ebib-reading-list-template'.
 Each specifier consists of a character (which is preceded by a
 percent sign in `ebib-reading-list-template') and a symbol, which
@@ -105,6 +106,20 @@ can be used in an org :PROPERTIES: block."
   :group 'ebib-reading-list
   :type '(string :tag "Done marker"))
 
+(defcustom ebib-reading-list-add-item-function 'ebib-reading-list-move-point-default
+  "Function to run when adding an item to the reading list.
+This function is run with point at the beginning of the buffer
+and should move point to the correct position in the buffer.
+
+The default function simply moves point to the end of the buffer."
+  :group 'ebib-reading-list
+  :type 'function)
+
+(defun ebib-reading-list-move-point-default ()
+  "Default value for `ebib-reading-list-add-item-function'.
+This function just moves point to the end of the buffer."
+  (goto-char (point-max)))
+
 (defcustom ebib-reading-list-remove-item-function 'ebib-reading-list-mark-item-as-done
   "Function to run when removing an item from the reading list.
 This function is run with point positioned after the item's
@@ -127,7 +142,7 @@ still pending, nil if it is done."
   :type 'function)
 
 (defun ebib-reading-list-item-org-active-p ()
-  "Return t if point is in a reading list item that is done."
+  "Return t if point is in a reading list item that is still active."
   (string= (org-get-todo-state) ebib-reading-list-todo-marker))
 
 (defcustom ebib-reading-list-new-item-hook nil
@@ -189,7 +204,7 @@ Return KEY.  If there is already an item for KEY, do nothing and
 return nil."
   (with-current-buffer (ebib--reading-list-buffer)
     (unless (ebib--reading-list-locate-item key)
-      (goto-char (point-max))
+      (funcall ebib-reading-list-add-item-function)
       (insert (ebib--reading-list-fill-template key db))
       (run-hooks 'ebib-reading-list-new-item-hook)
       (save-buffer)
@@ -200,11 +215,13 @@ return nil."
 Return KEY if the item was removed.  If there is no item for KEY,
 do nothing and return nil."
   (with-current-buffer (ebib--reading-list-buffer)
-    (when (ebib--reading-list-locate-item key)
-      (funcall ebib-reading-list-remove-item-function)
-      (run-hooks 'ebib-reading-list-remove-item-hook)
-      (save-buffer)
-      key)))
+    (let ((location (ebib--reading-list-locate-item key)))
+      (when location
+        (goto-char location)
+        (funcall ebib-reading-list-remove-item-function)
+        (run-hooks 'ebib-reading-list-remove-item-hook)
+        (save-buffer)
+        key))))
 
 (defun ebib--reading-list-fill-template (key db)
   "Create the text for a reading list item for KEY in DB."
